#include <assert.h>
#include <stdlib.h>
#include "uintset.h"

enum { bucket_size = 32 };

struct entry {
   unsigned int n;
   struct entry* next;
};

struct uintset {
   struct entry* bucket[bucket_size];
};

bool uintset_init(uintset set) {
   struct uintset* p = calloc(1, sizeof(struct uintset));
   set[0] = p;
   return p != 0;
}

bool uintset_add(uintset set, unsigned int member) {
   unsigned int index = member % bucket_size;
   struct entry* entry = set[0]->bucket[index];
   while (entry && entry->n != member) {
      entry = entry->next;
   }
   if (entry) return true; // is already a member
   entry = malloc(sizeof(struct entry));
   if (!entry) return false;
   entry->n = member;
   entry->next = set[0]->bucket[index]; set[0]->bucket[index] = entry;
   return true;
}

bool uintset_ismember(uintset set, unsigned int member) {
   unsigned int index = member % bucket_size;
   struct entry* entry = set[0]->bucket[index];
   while (entry && entry->n != member) {
      entry = entry->next;
   }
   return entry != 0;
}

void uintset_clear(uintset set) {
   for (int index = 0; index < bucket_size; ++index) {
      struct entry* entry = set[0]->bucket[index];
      while (entry) {
	 struct entry* old = entry;
	 entry = entry->next;
	 free(old);
      }
   }
   free(set[0]);
   set[0] = 0;
}
